Re: [PATCH 20/32] omap_hsmmc: put MMC regulator to sleep

2009-07-26 Thread Adrian Hunter

Andrew Morton wrote:

On Fri, 10 Jul 2009 15:42:27 +0300
Adrian Hunter adrian.hun...@nokia.com wrote:


+static int omap_mmc_regsleep_to_enabled(struct mmc_omap_host *host)
+{
+   unsigned long timeout;
+
+   dev_dbg(mmc_dev(host-mmc), REGSLEEP - ENABLED\n);
+
+   clk_enable(host-fclk);
+   clk_enable(host-iclk);
+
+   if (clk_enable(host-dbclk))
+   dev_dbg(mmc_dev(host-mmc),
+   Enabling debounce clk failed\n);
+
+   omap_mmc_restore_ctx(host);
+
+   /*
+* We turned off interrupts and bus power.  Interrupts
+* are turned on by 'mmc_omap_start_command()' so we
+* just need to turn on the bus power here.
+*/
+   OMAP_HSMMC_WRITE(host-base, HCTL,
+OMAP_HSMMC_READ(host-base, HCTL) | SDBP);
+
+   timeout = jiffies + msecs_to_jiffies(MMC_TIMEOUT_MS);
+   while ((OMAP_HSMMC_READ(host-base, HCTL)  SDBP) != SDBP 
+  time_before(jiffies, timeout))
+   ;
+
+   if (mmc_slot(host).set_sleep)
+   mmc_slot(host).set_sleep(host-dev, host-slot_id,
+0, host-vdd, 0);
+
+   host-dpm_state = ENABLED;
+
+   return 0;
+}


We take no action if the wait for SDBP timed out?


Yeah that is on my TODO list, but SDBP (and other bits we wait for)
will not timeout unless the hardware is broken or there is a bug
in the driver.  In either of those cases we are ruined anyway.

--
To unsubscribe from this list: send the line unsubscribe linux-omap in
the body of a message to majord...@vger.kernel.org
More majordomo info at  http://vger.kernel.org/majordomo-info.html


Re: [PATCH 20/32] omap_hsmmc: put MMC regulator to sleep

2009-07-24 Thread Andrew Morton
On Fri, 10 Jul 2009 15:42:27 +0300
Adrian Hunter adrian.hun...@nokia.com wrote:

 +static int omap_mmc_regsleep_to_enabled(struct mmc_omap_host *host)
 +{
 + unsigned long timeout;
 +
 + dev_dbg(mmc_dev(host-mmc), REGSLEEP - ENABLED\n);
 +
 + clk_enable(host-fclk);
 + clk_enable(host-iclk);
 +
 + if (clk_enable(host-dbclk))
 + dev_dbg(mmc_dev(host-mmc),
 + Enabling debounce clk failed\n);
 +
 + omap_mmc_restore_ctx(host);
 +
 + /*
 +  * We turned off interrupts and bus power.  Interrupts
 +  * are turned on by 'mmc_omap_start_command()' so we
 +  * just need to turn on the bus power here.
 +  */
 + OMAP_HSMMC_WRITE(host-base, HCTL,
 +  OMAP_HSMMC_READ(host-base, HCTL) | SDBP);
 +
 + timeout = jiffies + msecs_to_jiffies(MMC_TIMEOUT_MS);
 + while ((OMAP_HSMMC_READ(host-base, HCTL)  SDBP) != SDBP 
 +time_before(jiffies, timeout))
 + ;
 +
 + if (mmc_slot(host).set_sleep)
 + mmc_slot(host).set_sleep(host-dev, host-slot_id,
 +  0, host-vdd, 0);
 +
 + host-dpm_state = ENABLED;
 +
 + return 0;
 +}

We take no action if the wait for SDBP timed out?
--
To unsubscribe from this list: send the line unsubscribe linux-omap in
the body of a message to majord...@vger.kernel.org
More majordomo info at  http://vger.kernel.org/majordomo-info.html


[PATCH 20/32] omap_hsmmc: put MMC regulator to sleep

2009-07-10 Thread Adrian Hunter
From 66c91dd04118c2e2111dade9d5a327c442912046 Mon Sep 17 00:00:00 2001
From: Adrian Hunter adrian.hun...@nokia.com
Date: Tue, 12 May 2009 20:54:51 +0300
Subject: [PATCH] omap_hsmmc: put MMC regulator to sleep

When a card is not in use, the voltage regulator can be put
to sleep.  This is an alternative to powering the card off,
when powering off is not safe because the card might be
replaced without the driver being aware of it.

That situation happens if:
- the card is removable i.e. not eMMC
- and there is no card detect
- and there is a cover switch but the cover is open

Signed-off-by: Adrian Hunter adrian.hun...@nokia.com
---
 drivers/mmc/host/omap_hsmmc.c |   59 +---
 1 files changed, 54 insertions(+), 5 deletions(-)

diff --git a/drivers/mmc/host/omap_hsmmc.c b/drivers/mmc/host/omap_hsmmc.c
index c69ab89..eb301c1 100644
--- a/drivers/mmc/host/omap_hsmmc.c
+++ b/drivers/mmc/host/omap_hsmmc.c
@@ -160,6 +160,7 @@ struct mmc_omap_host {
int response_busy;
int context_loss;
int dpm_state;
+   int vdd;
 
struct  omap_mmc_platform_data  *pdata;
 };
@@ -1031,10 +1032,12 @@ static void omap_mmc_set_ios(struct mmc_host *mmc, 
struct mmc_ios *ios)
case MMC_POWER_OFF:
mmc_slot(host).set_power(host-dev, host-slot_id,
 0, 0);
+   host-vdd = 0;
break;
case MMC_POWER_UP:
mmc_slot(host).set_power(host-dev, host-slot_id,
 1, ios-vdd);
+   host-vdd = ios-vdd;
break;
case MMC_POWER_ON:
do_send_init_stream = 1;
@@ -1172,19 +1175,20 @@ static void omap_hsmmc_init(struct mmc_omap_host *host)
 
 /*
  * Dynamic power saving handling, FSM:
  *   ENABLED - DISABLED - OFF
  * ^___|  |
  * |__|
  *
  * ENABLED:   mmc host is fully functional
  * DISABLED:  fclk is off
  * OFF:   fclk is off,voltage regulator is off
+ * REGSLEEP:  fclk is off,voltage regulator is asleep
  *
  * Transition handlers return the timeout for the next state transition
  * or negative error.
  */
 
-enum {ENABLED = 0, DISABLED, OFF};
+enum {ENABLED = 0, DISABLED, REGSLEEP, OFF};
 
 /* Handler for [ENABLED - DISABLED] transition */
 static int omap_mmc_enabled_to_disabled(struct mmc_omap_host *host)
@@ -1221,8 +1225,12 @@ static int omap_mmc_disabled_to_off(struct mmc_omap_host 
*host)
 mmc_slot(host).get_cover_state(host-dev, host-slot_id))) {
mmc_power_save_host(host-mmc);
new_state = OFF;
-   } else
-   new_state = DISABLED;
+   } else {
+   if (mmc_slot(host).set_sleep)
+   mmc_slot(host).set_sleep(host-dev, host-slot_id,
+1, 0, 0);
+   new_state = REGSLEEP;
+   }
 
OMAP_HSMMC_WRITE(host-base, ISE, 0);
OMAP_HSMMC_WRITE(host-base, IE, 0);
@@ -1279,6 +1287,44 @@ static int omap_mmc_off_to_enabled(struct mmc_omap_host 
*host)
return 0;
 }
 
+/* Handler for [REGSLEEP - ENABLED] transition */
+static int omap_mmc_regsleep_to_enabled(struct mmc_omap_host *host)
+{
+   unsigned long timeout;
+
+   dev_dbg(mmc_dev(host-mmc), REGSLEEP - ENABLED\n);
+
+   clk_enable(host-fclk);
+   clk_enable(host-iclk);
+
+   if (clk_enable(host-dbclk))
+   dev_dbg(mmc_dev(host-mmc),
+   Enabling debounce clk failed\n);
+
+   omap_mmc_restore_ctx(host);
+
+   /*
+* We turned off interrupts and bus power.  Interrupts
+* are turned on by 'mmc_omap_start_command()' so we
+* just need to turn on the bus power here.
+*/
+   OMAP_HSMMC_WRITE(host-base, HCTL,
+OMAP_HSMMC_READ(host-base, HCTL) | SDBP);
+
+   timeout = jiffies + msecs_to_jiffies(MMC_TIMEOUT_MS);
+   while ((OMAP_HSMMC_READ(host-base, HCTL)  SDBP) != SDBP 
+  time_before(jiffies, timeout))
+   ;
+
+   if (mmc_slot(host).set_sleep)
+   mmc_slot(host).set_sleep(host-dev, host-slot_id,
+0, host-vdd, 0);
+
+   host-dpm_state = ENABLED;
+
+   return 0;
+}
+
 /*
  * Bring MMC host to ENABLED from any other PM state.
  */
@@ -1289,6 +1335,8 @@ static int omap_mmc_enable(struct mmc_host *mmc)
switch (host-dpm_state) {
case DISABLED:
return omap_mmc_disabled_to_enabled(host);
+   case REGSLEEP:
+   return omap_mmc_regsleep_to_enabled(host);
@@ -1172,19 +1175,20 @@ static void omap_hsmmc_init(struct mmc_omap_host *host)
 /*
  * Dynamic power saving handling, FSM:
- *   ENABLED - DISABLED - OFF
+ *   ENABLED - DISABLED - OFF / REGSLEEP
  * ^___|  |
  * |__|
case OFF:
return omap_mmc_off_to_enabled(host);
default: